%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the kernelised versions of the Proportional Odds
% Model presented in the paper: Kernelising the Proportional Odds Model
% through Kernel Learning techniques
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/neucom-kpom
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [projected,testTargets]= test_kPOM(testPatterns, model)
% Test the POM model constructed on testPatterns
% Input variables
% *testPatterns: Matrix containing the test patterns (a kernel matrix is 
%  to be provided if using the kernelized version of the algorithm)
% *model: struct obtained from the learning procedure containing the
% projection and the set of thresholds
% Output variables
% *projected: projected values for the test patterns provided
% *testTargets: vector with the prediction for the test patterns

         numClasses = size(model.thresholds,1)+1;
         projected = model.projection' * testPatterns';
                
         % Compute projected patterns
         projected2 = repmat(projected, numClasses-1,1);
         projected2 = projected2 - model.thresholds*ones(1,size(projected2,2));

         % Asignation of the class label
         % f(x) = max {Wx-bk<0} or Wx - b_(K-1) > 0
         wx=projected2;
         % We assign the values > 0 to NaN
         wx(wx(:,:)>0)=NaN;
         % Then, we choose the bigger one.
         [maximum,testTargets]=max(wx,[],1);

         % If the maximum is equal to NaN is because Wx-bk for all k
         % is >0, so the pattern belows to the last class.
         testTargets(isnan(maximum(:,:)))=numClasses;

         testTargets = testTargets';
end
