%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the kernelised versions of the Proportional Odds
% Model presented in the paper: Kernelising the Proportional Odds Model
% through Kernel Learning techniques
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/neucom-kpom
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [align, valuederivative] = alignmentLMIKDerivative( param, eigenval, eigenvec, k_ic, lambda)
% Compute the derivative of the regularised alignment-based function to 
% optimise the dimensions of the EFS 
% Input variables
%  *param: weights for each of the dimensions of the EFS
%  *eigenval: eigenvalues for the kernel matrix
%  *eigenvec: eigenvectors for the kernel matrix
%  *k_ic: centered ideal kernel matrix
%  *lambda: lambda parameter for the regularisation
% Output variables
%  *align: value for the fucntion considered for the optimisation
%  *valuederivative: derivative value for the param solution

    k = eigenvec * diag(sigmoid(param)) * eigenval * eigenvec';
    meank = (eye(size(k,1)) - (ones(size(k,1),1)*ones(1,size(k,1)))/size(k,1));
    k_c = (meank*k*meank')/trace((meank*k*meank'));
    n2k_c = sqrt(sum(sum((k_c).*k_c)));
    n2k_ic = sqrt(sum(sum(k_ic.*k_ic)));
    k_ck_ic = (sum(sum(k_c.*k_ic)));

    for i=1:numel(param),
        kd = 2*sigmoid(param(i)).*(1-sigmoid(param(i)))*eigenval(i,i)*eigenvec(:,i)*(eigenvec(:,i)');
        valuederivative(i) = -(1/(n2k_ic)) .* ( ((sum(sum(k_ic.*kd)))/n2k_c) - ((k_ck_ic.*(sum(sum(k_c.*kd))))/(n2k_c^(3)))) + (lambda/numel(param))*sigmoid(param(i)).*(1-sigmoid(param(i)));
    end

    align = -(k_ck_ic/(n2k_c .* n2k_ic)) + lambda*sum(sigmoid(param))/numel(param);
end
