%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the kernelised versions of the Proportional Odds
% Model presented in the paper: Kernelising the Proportional Odds Model
% through Kernel Learning techniques
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/neucom-kpom
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function kernelMatrix  = computeKernelMatrix(patterns1, patterns2, kernelParam)
% Compute the kernel matrix between two sets of patterns using the Gaussian
% kernel funcction
% *Inputs:
%   -patterns1, patterns2: Two matrices of patterns
%   -kernelParam: Kernel width parameter
% *Outputs:
%   -kernelMatrix: Resulting kernel matrix

d = pdist2(patterns1,patterns2);
kernelMatrix = exp(-kernelParam*d.^2);
        
end
