%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the kernelised versions of the Proportional Odds
% Model presented in the paper: Kernelising the Proportional Odds Model
% through Kernel Learning techniques
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/neucom-kpom
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Example code for a toy ordinal dataset.
load toy-train
load toy-test

% Example of sigma values to test when optimising a combination of Gaussian
% kernels
example_kernelSigma = [10^-3, 10^-2, 10^-1, 10^0, 10^1, 10^2, 10^3];
param.kernelSigma = example_kernelSigma;
% Whether to use the original POM algorithm or the kernelized version
options.kernelized = 1;
% Combination of Gaussian kernels on
options.kernelLearning = 1;
% Include the ordinal weight matrix (for the optimisation of the combination
% of kernels or for the gradient based technique for optimising the EFS 
% dimensions)
options.ordinal = 1;
% Optimising the dimensions chosen for the EFS by the proposed gradient 
% ascent technique
options.regularisedGradientBased = 1;
% Lambda parameter that controls the amount of regularisation to apply when
% optimising the dimensions
param.lambda = 0.1;
% Number of dimensions chosen for the EFS (only applies when not explicitely
% optimising the number of dimensions)
param.EFSdimensions = 10;

% Optimising the combination of kernels
if options.kernelLearning & options.kernelized
    param.kernelWeight = optimiseCombinationOfKernels(train,example_kernelSigma,options.ordinal);
end
% Run the kPOM method with the selected configuration
[model_information, mae_error] = run_kPOM(train, test, param, options);
disp(['Obtained MAE is ' num2str(mae_error)])
